#include <romfs/romfs.hpp>

#include <map>
#include <string>
#include <filesystem>
#include <vector>

#include <cstddef>

/* File embedder macros */
#if defined(__clang__) && !defined(__APPLE__)

    #define RESOURCE(name, path)                        \
    __asm__ (                                           \
        ".global " #name "\n"                           \
        ".global " #name "_size\n"                      \
        #name ":\n"                                     \
            ".incbin \"" path "\"\n"                    \
            ".byte 0\n"                                 \
            ".align 8\n"                                \
        #name "_size:\n"                                \
            ".int " #name "_size - " #name " - 1\n"     \
            ".align 8\n"                                \
    )

#elif defined(__APPLE__)

    #define RESOURCE(name, path)                        \
    __asm__ (                                           \
        ".global _" #name "\n"                          \
        ".global _" #name "_size\n"                     \
        "_" #name ":\n"                                 \
            ".incbin \"" path "\"\n"                    \
            ".byte 0\n"                                 \
            ".align 8\n"                                \
        "_" #name "_size:\n"                            \
            ".int _" #name "_size - _" #name " - 1\n"   \
            ".align 8\n"                                \
    )


#elif defined(__GNUC__)

    #if defined(__linux__)
        #define RESOURCE(name, path)                    \
        __asm__ (                                       \
            ".section .rodata\n"                        \
            #name ":\n"                                 \
                ".incbin \"" path "\"\n"                \
                ".byte 0\n"                             \
                ".type " #name ", @object\n"            \
                ".size " #name "_size, 1\n"             \
            #name "_size:\n"                            \
                ".int " #name "_size - " #name " - 1\n" \
                ".align 8\n"                            \
        )
    #else
        #define RESOURCE(name, path)                    \
        __asm__ (                                       \
            ".section .rodata\n"                        \
            #name ":\n"                                 \
                ".incbin \"" path "\"\n"                \
                ".byte 0\n"                             \
            #name "_size:\n"                            \
                ".int " #name "_size - " #name " - 1\n" \
                ".align 8\n"                            \
        )
    #endif

#else
    #error "Unknown compiler!"
#endif

#define RESOURCE_EXPORT(name)       \
    extern "C" std::byte name[];    \
    extern "C" int name##_size;


/* Resource definitions */
RESOURCE(resource0, "/builddir/build/BUILD/ImHex/resources/romfs/backdrop_light.png");
RESOURCE_EXPORT(resource0);
RESOURCE(resource1, "/builddir/build/BUILD/ImHex/resources/romfs/backdrop_dark.png");
RESOURCE_EXPORT(resource1);
RESOURCE(resource2, "/builddir/build/BUILD/ImHex/resources/romfs/banner_light.png");
RESOURCE_EXPORT(resource2);
RESOURCE(resource3, "/builddir/build/BUILD/ImHex/resources/romfs/splash.png");
RESOURCE_EXPORT(resource3);
RESOURCE(resource4, "/builddir/build/BUILD/ImHex/resources/romfs/cacert.pem");
RESOURCE_EXPORT(resource4);
RESOURCE(resource5, "/builddir/build/BUILD/ImHex/resources/romfs/logo.png");
RESOURCE_EXPORT(resource5);
RESOURCE(resource6, "/builddir/build/BUILD/ImHex/resources/romfs/banner_dark.png");
RESOURCE_EXPORT(resource6);
RESOURCE(resource7, "/builddir/build/BUILD/ImHex/resources/romfs/LICENSE");
RESOURCE_EXPORT(resource7);

struct RomFs {
    static std::map<std::filesystem::path, romfs::Resource> resources;
    static std::vector<std::filesystem::path> paths;
};

/* Resource map */
std::map<std::filesystem::path, romfs::Resource> RomFs::resources = {
    { "backdrop_light.png", romfs::Resource({ resource0, size_t(resource0_size) }) },
    { "backdrop_dark.png", romfs::Resource({ resource1, size_t(resource1_size) }) },
    { "banner_light.png", romfs::Resource({ resource2, size_t(resource2_size) }) },
    { "splash.png", romfs::Resource({ resource3, size_t(resource3_size) }) },
    { "cacert.pem", romfs::Resource({ resource4, size_t(resource4_size) }) },
    { "logo.png", romfs::Resource({ resource5, size_t(resource5_size) }) },
    { "banner_dark.png", romfs::Resource({ resource6, size_t(resource6_size) }) },
    { "LICENSE", romfs::Resource({ resource7, size_t(resource7_size) }) },
};

/* Resource paths */
std::vector<std::filesystem::path> RomFs::paths = {
    "backdrop_light.png",
    "backdrop_dark.png",
    "banner_light.png",
    "splash.png",
    "cacert.pem",
    "logo.png",
    "banner_dark.png",
    "LICENSE",
};

